"""
This file contains unit tests for bonsai config library.
"""
import os
import tempfile
from unittest import TestCase
from unittest.mock import patch

from six.moves.configparser import ConfigParser

from bonsai_config import BonsaiConfig


class TestBrainConfig(TestCase):

    def setUp(self):
        self.temp_path = os.path.join(tempfile.gettempdir(), 'test-bonsai')
        patcher = patch('bonsai_config.bonsai_config.CONFIG_FILE',
                        new=self.temp_path)

        self.addCleanup(patcher.stop)
        patcher.start()

    def tearDown(self):
        os.remove(self.temp_path)

    def test_url_pieces(self):
        cp = ConfigParser()
        cp['DEFAULT']['HOST'] = 'local.bons.ai'
        cp['DEFAULT']['PORT'] = '12345'
        cp['DEFAULT']['USESSL'] = 'False'
        with open(self.temp_path, 'w') as f:
            cp.write(f)

        bc = BonsaiConfig()
        self.assertEqual('http://local.bons.ai:12345', bc.brain_api_url())

    def test_url(self):
        cp = ConfigParser()
        cp['DEFAULT']['URL'] = 'https://local.bons.ai:3456'
        with open(self.temp_path, 'w') as f:
            cp.write(f)

        bc = BonsaiConfig()
        self.assertEqual('https://local.bons.ai:3456', bc.brain_api_url())

    def test_wss(self):
        cp = ConfigParser()
        cp['DEFAULT']['URL'] = 'https://local.bons.ai:3456'
        with open(self.temp_path, 'w') as f:
            cp.write(f)

        bc = BonsaiConfig()
        self.assertEqual('wss://local.bons.ai:3456', bc.brain_websocket_url())

    def test_section(self):
        cp = ConfigParser()
        cp['DEFAULT']['USERNAME'] = 'bonsai'
        cp['DEFAULT']['PROFILE'] = 'public'
        cp.add_section('public')
        cp['public']['USERNAME'] = 'override'

        with open(self.temp_path, 'w') as f:
            cp.write(f)

        bc = BonsaiConfig()
        self.assertEqual('override', bc.username())

    def test_fallback(self):
        cp = ConfigParser()
        cp['DEFAULT']['USERNAME'] = 'bonsai'
        cp['DEFAULT']['PROFILE'] = 'public'
        cp.add_section('public')
        cp['public']['OTHER'] = 'value'

        with open(self.temp_path, 'w') as f:
            cp.write(f)

        bc = BonsaiConfig()
        self.assertEqual('bonsai', bc.username())

    def test_profile(self):
        cp = ConfigParser()
        cp['DEFAULT']['USERNAME'] = 'bonsai'
        cp['DEFAULT']['HOST'] = 'local.bons.ai'

        with open(self.temp_path, 'w') as f:
            cp.write(f)

        bc = BonsaiConfig(section='test')
        bc.update(HOST='override')
        self.assertEqual('override', bc._host())
        self.assertEqual('bonsai', bc.username())

        bc.update(Profile='DEFAULT')
        self.assertEqual('local.bons.ai', bc._host())

